from ..textual_element import TextualElement


class LoadingIndicator(TextualElement):
    """
    A textual LoadingIndicator object (see: https://textual.textualize.io/widgets/loading_indicator/)
    """
