from .textual_element_bridge import TextualElementBridge
from .textual_fully_reidentifiable_element_bridge import TextualFullyReidentifiableElementBridge
from .textual_not_reidentifiable_element_bridge import TextualNotReidentifiableElementBridge
from .textual_partly_reidentifiable_element_bridge import TextualPartlyReidentifiableElementBridge


__all__ = [
    'TextualElementBridge',
    'TextualFullyReidentifiableElementBridge',
    'TextualNotReidentifiableElementBridge',
    'TextualPartlyReidentifiableElementBridge',
]
