from ...textual_element import TextualElement


class ContentSwitcher(TextualElement):
    """
    A textual ContentSwitcher object (see: https://textual.textualize.io/widgets/content_switcher/)
    """
